package com.hywa.map.common.utils;

import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * 日期工具类
 */
public class DateUtils {

    /**
     * 默认格式（2016-08-20）
     */
    public static final String DEFAULT_FORMAT = "yyyy-MM-dd";
    public static SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
    /**
     * 年月（2016-08）
     */
    public static final String YEAR_MONTH = "yyyy-MM";

    /**
     * 详细日期格式（2016-08-20 17:30:30）
     */
    public static final String DETAIL = "yyyy-MM-dd HH:mm:ss";

    /**
     * 带/的日期格式（2016/08/20）
     */
    public static final String FORMAT_SLASH = "yyyy/MM/dd";

    /**
     * 中文的日期格式（2016年08月20日）
     */
    public static final String CHINESE_FORMAT = "yyyy年MM月dd日";

    public static final String CHINESE_FORMAT_DETAIL = "yyyy年MM月dd日 HH:mm:ss";

    /*一天 时间毫秒数*/
    public static final long ONE_DAY_MILLIS = 1000 * 3600 * 24;

    /**
     * 将String的日期转换为字符串中文格式
     */
    public static String formatChineseDate(String date) {
        try {
            if (null == date) {
                return null;
            }
            Date dateStr = stringToDate(date, DETAIL);
            SimpleDateFormat sdf = new SimpleDateFormat(CHINESE_FORMAT);
            return sdf.format(dateStr);
        } catch (ParseException e) {
            return null;
        }
    }

    /**
     * 将Date类型转换为字符串类型
     */
    public static String formatDate(Date date, String format) {
        if (null == date) {
            return null;
        }
        if (StringUtils.isEmpty(format)) {
            format = DEFAULT_FORMAT;
        }
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        return sdf.format(date);
    }

    /**
     * 将String类型转换为date类型
     */
    public static Date stringToDate(String dateStr, String format) throws ParseException {
        if (StringUtils.isEmpty(dateStr)) {
            return null;
        }
        if (StringUtils.isEmpty(format)) {
            format = DEFAULT_FORMAT;
        }
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        return sdf.parse(dateStr);
    }

    /**
     * 得到本月最开始的时间
     */
    public static Date currentMonthStart() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        return calendar.getTime();
    }

    /**
     * 得到本月最后一刻时间
     */
    public static Date currentMonthEnd() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        return calendar.getTime();
    }

    /**
     * 得到本年年份
     */
    public static int currentYear() {
        Calendar calendar = Calendar.getInstance();
        return calendar.get(Calendar.YEAR);
    }

    /**
     * 通过生日得到年龄
     */
    public static int getAge(Date birthDate) {
        if (birthDate == null) {
            throw new RuntimeException("出生日期不能为null");
        }
        int age = 0;
        Date now = new Date();
        SimpleDateFormat formatYear = new SimpleDateFormat("yyyy");
        SimpleDateFormat formatMonth = new SimpleDateFormat("MM");
        String birthYear = formatYear.format(birthDate);
        String thisYear = formatYear.format(now);
        String birthMonth = formatMonth.format(birthDate);
        String thisMonth = formatMonth.format(now);
        // 初步，估算
        age = Integer.parseInt(thisYear) - Integer.parseInt(birthYear);
        // 如果未到出生月份，则age - 1
        if (thisMonth.compareTo(birthMonth) < 0) {
            age -= 1;
        }
        if (age < 0) {
            age = 0;
        }
        return age;
    }

    /**
     * 通过日期生成惟一主键
     */
    public static synchronized String generateId() {
        return System.currentTimeMillis() + "";
    }

    public static synchronized String generateDateId() {
        Date date = new Date();
        String dateStr = formatDate(date, "yyyyMMddHHmmss");
        return dateStr;
    }

    /**
     * 功能：获取当前日期 格式:20101010
     *
     * @return
     */
    public static String getCurrentDate() {
        Date date = new Date();
        return dateFormat.format(date);
    }

    /**
     * 当前时间加 减days
     *
     * @param days
     * @return yyyy-MM-dd
     */
    public static String getDiffDay(int days) {
        SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd");
        Calendar c = Calendar.getInstance();
        c.add(Calendar.DAY_OF_MONTH, days);
        return sf.format(c.getTime());
    }

    /**
     * 功能：返回小时
     *
     * @param date
     * @return
     */
    public static int getHour(Date date) {
        if (date == null) {
            date = new Date();
        }
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        return c.get(Calendar.HOUR_OF_DAY);
    }

    /**
     * 获取指定日期中的月份日期
     *
     * @return String 如12月21日
     */
    public static String getMDChina(String dateStr) {
        String str = newLongYMDFormat().format(parseDate(dateStr));
        return str.split("-")[1] + "月" + str.split("-")[2] + "日";
    }

    /**
     * 创建一个"yyyy-MM-dd"日期的格式化对象
     *
     * @return "yyyy-MM-dd"日期的格式化对象
     */
    private static SimpleDateFormat newLongYMDFormat() {
        return new SimpleDateFormat("yyyy-MM-dd");
    }

    /**
     * 功能：解析数据库中的日期字符串 格式:yyyy-MM-dd
     *
     * @param dateStr
     * @return
     */
    public static Date parseDate(String dateStr) {
        Date date = null;
        try {
            date = dateFormat.parse(dateStr);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }

    /**
     * 获取指定日期是星期几<br>
     *
     * @param dt
     * @return 指定日期是星期几
     */
    public static String getWeek(String dt) {
        String[] weekDays = {"周日", "周一", "周二", "周三", "周四", "周五", "周六"};
        Calendar cal = Calendar.getInstance();
        cal.setTime(parseDate(dt));
        int w = cal.get(Calendar.DAY_OF_WEEK) - 1;
        if (w < 0) {
            w = 0;
        }
        return weekDays[w];
    }

    public static void main(String[] args) {
        System.out.println(formatDate(currentMonthStart(), DEFAULT_FORMAT));
        System.out.println(getCurrentDate());
        System.out.println(String.format("%.2f", (double) 32));
        System.out.println(getDiffDay(-5));
        System.out.println(1+(1/2)+(1/3)+(1/4));
        String value="112.00";
        DecimalFormat decimalFormat = new DecimalFormat("###################.###########");

        System.out.println(currentYear());
        String test = "2019 年2月6日".replaceAll(" ","");
        String year = test.substring(0,4);
        String month=test.substring(test.indexOf("年")+1,test.indexOf("月"));
        String day=test.substring(test.indexOf("月")+1,test.indexOf("日"));
        month=month.length()==1?"0"+month:month;
        day=day.length()==1?"0"+day:day;
        System.out.println(year+"==="+month+"==="+day);
    }
}
