package com.hywa.map.controller.login;

import com.hywa.map.common.ConstantPool;
import com.hywa.map.common.enums.CommonEnum;
import com.hywa.map.common.exception.GlobalException;
import com.hywa.map.common.log.Log;
import com.hywa.map.common.utils.FileUtils;
import com.hywa.map.common.utils.SpringContext;
import com.hywa.map.common.utils.StringUtils;
import com.hywa.map.config.EnvProperties;
import com.hywa.map.config.interceptor.TokenManager;
import com.hywa.map.config.interceptor.TokenModel;
import com.hywa.map.config.validation.JsonValidate;
import com.hywa.map.controller.login.validate.StateValidationValidate;
import com.hywa.map.dao.entity.User;
import com.hywa.map.dao.mapper.UserMapper;
import com.hywa.map.representation.*;
import com.hywa.map.representation.ext.UserRepExt;
import com.hywa.map.service.login.MenuService;
import com.hywa.map.service.login.UserRoleService;
import com.hywa.map.service.login.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.net.util.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.util.*;

import static com.hywa.map.common.ConstantPool.SPLIT;
import static com.hywa.map.common.enums.CommonEnum.*;

@Slf4j
@Api(tags = "用户管理")
@RestController
@RequestMapping("user")
public class UserController {

    private static final String MOBILE = "mobile";
    private static final String PC = "pc";
    private static final String SPOT = "spot";

    @Autowired
    EnvProperties envProperties;

    @Autowired
    TokenManager tokenManager;

    @Autowired
    private UserService userService;

    @Autowired
    private MenuService menuService;

    @Autowired
    UserRoleService userRoleService;


    @ApiOperation(value = "移动端登录")
    @PostMapping(value = "/login")
    @Log(log = "登录-移动端用户登录系统")
    //@JsonValidate(StateValidationValidate.class)
    public ResponseEntity<ResponseBase> login(
            @RequestBody LoginInfoRep loginInfoRep,
            HttpServletRequest req) throws UnsupportedEncodingException {
        validate(loginInfoRep);
        String pwd = checkoutPwd(loginInfoRep.getPwd());
        ResponseBase<UserRep> userRepResponseBase = getUserRepResponseBase(
                loginInfoRep.getUserName(), pwd, req, MOBILE);
        return new ResponseEntity<>(userRepResponseBase, HttpStatus.OK);
    }

    @ApiOperation(value = "PC端登录")
    @PostMapping(value = "/pcLogin")
    @Log(log = "登录-用户登录系统")
    public ResponseEntity<ResponseBase> pcLogin(
            @RequestBody LoginInfoRep loginInfoRep,
            HttpServletRequest req) throws UnsupportedEncodingException {
        validate(loginInfoRep);
        String pwd = checkoutPwd(loginInfoRep.getPwd());
        ResponseBase<UserRep> userRepResponseBase = getUserRepResponseBase(loginInfoRep.getUserName(), pwd, req, PC);
        return new ResponseEntity<>(userRepResponseBase, HttpStatus.OK);
    }


    private ResponseBase<UserRep> getUserRepResponseBase(String userName,
                                                         String password, HttpServletRequest req, String equipment) {
        ResponseBase<UserRep> userRepResponseBase = new ResponseBase<>();
        UserRep userRep = userService.userLogin(userName, password);
        userService.setAuthority(userRep.getUserId());
        TokenModel token = tokenManager.createToken(equipment + SPLIT + userRep.getUserId());
        req.getSession().setAttribute("userId", userRep.getUserId());
        userRep.setAccessToken(equipment + SPLIT + userRep.getUserId() + "_" + token.getToken());
        userRep.setSysReps(userService.findSysInfoByUserId(userRep.getUserId(), userRep.getName()));
        userRepResponseBase.success();
        userRepResponseBase.setRetBody(userRep);
        return userRepResponseBase;
    }

    @GetMapping(value = "/checkToken")
    public ResponseEntity<ResponseBase> tokenVerify(
            @RequestParam String userId,
            @RequestParam String token) {
        ResponseBase<UserRep> userRepResponseBase = checkUserToken(userId, token, MOBILE);
        return new ResponseEntity<>(userRepResponseBase, HttpStatus.OK);
    }

    @GetMapping(value = "/pcCheckToken")
    public ResponseEntity<ResponseBase> pcTokenVerify(
            @RequestParam String userId,
            @RequestParam String token) {
        ResponseBase<UserRep> userRepResponseBase = checkUserToken(userId, token, PC);
        return new ResponseEntity<>(userRepResponseBase, HttpStatus.OK);
    }

    private ResponseBase<UserRep> checkUserToken(String userId, String token, String equipment) {
        String[] split = token.split("_");
        boolean result = tokenManager
                .checkToken(new TokenModel(equipment + SPLIT + userId, split[1]));
        ResponseBase<UserRep> userRepResponseBase = new ResponseBase<>();
        if (result) {
            UserRep userRep = userService.findById(userId);
            userRep.setAccessToken(token);
            userRepResponseBase.setRetBody(userRep);
            userRepResponseBase.success();
        } else {
            userRepResponseBase.setRetHead(RE_LOGIN.getIndex(), RE_LOGIN.getValue());
        }
        return userRepResponseBase;
    }


    @PostMapping
    @Transactional
    @Log(log = "账户管理-新增账户")
    public ResponseEntity<ResponseBase> registerUser(@Valid @RequestBody UserRepExt userRep,
                                                     HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        if (StringUtils.isEmpty(userRep.getPwd())) {
            throw new GlobalException(ILLEGAL_PARAMETER_ERROR.getIndex(),
                    ILLEGAL_PARAMETER_ERROR.getValue() + ",密码为空",userId,"账户管理-新增账户");
        }
        String roleIds = userRep.getRoleIds();
        List<String> roles = new ArrayList<>();
        roles.add(roleIds);
        userRep.setRoles(roles);
        userRep.setCreateUser(userId);
        userService.save(userRep);
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/logout", method = RequestMethod.DELETE)
    public ResponseEntity<ResponseBase> logout(@RequestParam String userId) {
        tokenManager.deleteToken(MOBILE + SPLIT + userId);
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);

    }

    @RequestMapping(value = "/pcLogout", method = RequestMethod.DELETE)
    public ResponseEntity<ResponseBase> pcLogout(@RequestParam(value = "userId") String userId) {
        tokenManager.deleteToken(PC + SPLIT + userId);
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);

    }

    @DeleteMapping
    @Log(log = "账户管理-删除账户")
    public ResponseEntity<ResponseBase> deleteUser(@RequestParam String userId,
                                                   HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        checkUpdateInfo(userId, accessToken.getUserId().split(SPLIT)[1],"账户管理-删除账户");
        userService.delete(userId);
        //用户删除时，删除token
        deleteTokenByUserId(userId);
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/findAll", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase> findAllUser(
            @RequestParam(required = false) String name,
            @RequestParam(required = false) String region,
            @RequestParam(required = false) String workUnit,
            @RequestParam(defaultValue = "1") int pageNo,
            @RequestParam(defaultValue = "10") int pageSize, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        //Map<String, Object> userMap = userService.findAllById(userId);
        //String userName = (String) userMap.get("name");
        ResponseBase<UserReps> responseBase = new ResponseBase<>();
        responseBase.success()
                .setRetBody(userService
                        .findUsers(pageNo, pageSize, name, region, workUnit, null, userId));
        return new ResponseEntity<>(responseBase, HttpStatus.OK);

    }

    @RequestMapping(value = "/findByName", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase<User>> findByName(String name) {
        ResponseBase<User> responseBase = new ResponseBase<>();
        responseBase.success()
                .setRetBody(userService.findByName(name));
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @RequestMapping(value = "/findById", method = RequestMethod.GET)
    public ResponseEntity<ResponseBase<Map<String, Object>>> findAllById(String id)
            throws IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        ResponseBase<Map<String, Object>> responseBase = new ResponseBase<>();
        responseBase.success()
                .setRetBody(userService.findAllById(id));
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @PutMapping
    @Log(log = "账户管理-修改账户")
    public ResponseEntity<ResponseBase> update(
            @RequestBody UserRepExt userRep, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        String roleIds = userRep.getRoleIds();
        List<String> roles = new ArrayList<>();
        roles.add(roleIds);
        userRep.setRoles(roles);
        checkUpdateInfo(userRep.getUserId(), userId,"账户管理-修改账户");
        ResponseBase responseBase = new ResponseBase<>();
        String pwd = userRep.getPwd();
        userRep.setPwd(StringUtils.isNotEmpty(pwd) ? StringUtils.md5(pwd) : null);
        userService.update(userRep, userId);
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @PutMapping("updatePersonInfo")
    @Log(log = "账户管理-个人资料")
    public ResponseEntity<ResponseBase> updatePersonInfo(
            @RequestBody UserRepExt userRep, HttpServletRequest req) {
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        String userId = accessToken.getUserId().split(SPLIT)[1];
        ResponseBase responseBase = new ResponseBase<>();
        userService.updatePersonInfo(userRep, userId);
        responseBase.success();
        return new ResponseEntity<>(responseBase, HttpStatus.OK);
    }

    @PutMapping("changePwd")
    public ResponseEntity changePwd(@RequestBody @Valid NewUserInfoRep newUserInfoRep) {
        if (newUserInfoRep.getNewpassword().equals(newUserInfoRep.getPassword())) {
            throw new GlobalException(PASSWORD_REPEAT.getIndex(), PASSWORD_REPEAT.getValue(), null, "系统设置-修改密码");
        }
        userService.changePwd(newUserInfoRep);
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @PutMapping("resetPwd")
    public ResponseEntity resetPwd(@RequestBody NewUserInfoRep newUserInfoRep) {
        newUserInfoRep.setNewpassword("123456");
        userService.resetPwd(newUserInfoRep);
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }

    @RequestMapping(value = "/disableUser", method = RequestMethod.POST)
    @Log(log = "账户管理-禁用账户")
    public ResponseEntity<ResponseBase<User>> disableUser(@RequestBody UserRepExt userRep, HttpServletRequest req) {
        String menuName = "账户管理-禁用账户";
        TokenModel accessToken = tokenManager.getToken(req.getHeader("accessToken"));
        Map<String, String> map = new HashMap();
        String updateUserId=accessToken.getUserId().split(SPLIT)[1];
        String userId=userRep.getUserId();
        map.put("status", "0".equals(userRep.getStatus()) ? "1" : "0");
        map.put("updateUserId", updateUserId);
        map.put("userId", userId);
        if (envProperties.getAdminId().equals(userId)) {
            throw new GlobalException(CommonEnum.DISABLE_ADMIN.getIndex(),
                    CommonEnum.DISABLE_ADMIN.getValue(),null,menuName);
        }
        if (updateUserId.equals(userId)) {
            throw new GlobalException(CommonEnum.DISABLE_SELF.getIndex(),
                    CommonEnum.DISABLE_SELF.getValue(),null,menuName);
        }
        userService.disableUser(map);
        //用户禁用时，删除token
        deleteTokenByUserId(userId);
        return new ResponseEntity<>(new ResponseBase().success(), HttpStatus.OK);
    }


    private void checkUpdateInfo(@RequestParam String userId, String currentUserId,String menuName) {
        if (currentUserId.equals(userId)) {
            throw new GlobalException(CommonEnum.DEL_CURRENT_LOGIN_INFO.getIndex(),
                    CommonEnum.DEL_CURRENT_LOGIN_INFO.getValue(),null,menuName);
        }
        if (envProperties.getAdminId().equals(userId)) {
            throw new GlobalException(CommonEnum.DEL_ADMIN.getIndex(),
                    CommonEnum.DEL_ADMIN.getValue(),null,menuName);
        }
    }

    private String checkoutPwd(String pwd) throws UnsupportedEncodingException {
        //需要进行对应的接密操作
        //1.获取盐值
        String salt = pwd.substring(0, 5);
        String key = "b8d11ee289394be688ef3a4f6968efed";
        String time1 = StringUtils.md5((key).substring(0, 8)).substring(0, 5);
        //做超时判断
        if (!time1.equalsIgnoreCase(salt)) {
            throw new GlobalException(CommonEnum.LOGIN_TIMEOUT.getIndex(),
                    CommonEnum.LOGIN_TIMEOUT.getValue());
        }
        //2.去除盐值
        String replace = pwd.replace(salt, "");
        //3.还原base64前的密码值
        return new String(Base64.decodeBase64(replace), "utf-8");
    }

    /*
     * 用户禁用、删除时，将用户的token删除
     */
    public void deleteTokenByUserId(String userId) {
        try {
            //tokenManager.deleteToken(MOBILE + SPLIT + userId);
            tokenManager.deleteToken(PC + SPLIT + userId);
            //tokenManager.deleteToken(SPOT + SPLIT + userId);
        } catch (Exception e) {

        }
    }
    public void validate(LoginInfoRep loginInfoRep) {
        String menuName="用户登录-登录";
        ResponseBase responseBase = new ResponseBase<>();
        String userName=loginInfoRep.getUserName();
        if(Objects.equals(userName,"admin")){
            return ;
        }
        UserMapper userMapper = (UserMapper) SpringContext.getBean("userMapper");

        User user = userMapper.selectByUserName(userName);
        if(null==user){
            throw new GlobalException(CommonEnum.USER_UNREGISTERED_ERROR.getIndex(), CommonEnum.USER_UNREGISTERED_ERROR.getValue(),userName, menuName);
        }
        if (Objects.equals(ConstantPool.DISAABLE,user.getStatus())){
            throw new GlobalException(CommonEnum.THE_ACCOUNT_IS_BANNED.getIndex(), CommonEnum.THE_ACCOUNT_IS_BANNED.getValue(),userName, menuName);
        }

        Map<String, Integer> examine = userMapper.examine(userName);

        if (null==examine){
            throw new GlobalException(CommonEnum.USER_UNREGISTERED_ERROR.getIndex(), CommonEnum.USER_UNREGISTERED_ERROR.getValue(),userName, menuName);
        }

        switch (examine.get("roleInfo")){
            case 0:
                break;
            case 1:
            throw new GlobalException(CommonEnum.ABNORMAL_ACCOUNT.getIndex(),"该账户所属角色被锁定",userName, menuName);
            case 2:
            throw new GlobalException(CommonEnum.ABNORMAL_ACCOUNT.getIndex(),"该账户所属角色被禁用",userName, menuName);
            default:
                break;
        }
    }

}


